@@ -1,8 +1,9 @@
 import akiwi.app as kiwi
+import sys
 
 def main():
     app = kiwi.Application()
-    return app.run_with_command()
+    return 0 if app.run_with_command() else 100
 
 if __name__ == "__main__":
-    main()
\ No newline at end of file
+    sys.exit(main())
\ No newline at end of file
