@@ -4,6 +4,7 @@
 
 import sys
 import argparse
+import re
 import os
 import json
 import shutil
@@ -99,11 +100,12 @@ class Config:
             "DATA_DIR", "CODE_DIR", "PKG_DIR", "ACCESS_TOKEN", "SERVER", "CURRENT_ENV"
         ]
         self.INSTALLED_LIBS = {}
-        self.VENV           = {"libs":{}, "macros":{}}
+        self.VENV           = {"libs":{}, "macros":{}, "envars":{}}
         self.setup()
 
         self.CURRENT_MACROS = self.VENV["macros"]
         self.CURRENT_LIBS   = self.VENV["libs"]
+        self.CURRENT_ENVARS = self.VENV["envars"]
         vars_map = self.get_dict()
         for key in vars_map:
             os.environ["KIWI_" + key] = vars_map[key]
@@ -187,7 +189,7 @@ class Config:
             return True
 
         env_file  = os.path.join(self.VENV_DIR, f"{name}.json")
-        env_dict  = {"libs": {}, "macros": {}}
+        env_dict  = {"libs": {}, "macros": {}, "envars": {}}
         try:
             with open(env_file, "w") as f:
                 json.dump(env_dict, f)
@@ -197,7 +199,14 @@ class Config:
         return False
 
     def activate_env(self, name):
-        
+
+        if name is None:
+            name = self.CURRENT_ENV
+
+        if not utils.shellposix:
+            utils.gprint("You can not execute activate directly, you must use the posix shell to make the environment variables take effect.")
+            return False
+
         env_json  = os.path.join(self.VENV_DIR, f"{name}.json")
         if not os.path.exists(env_json):
             utils.gprint(f"Not exists virtual enviroment: {name}")
@@ -213,15 +222,61 @@ class Config:
         self.CURRENT_ENV = name
         self.CURRENT_MACROS = self.VENV.get("macros", {})
         self.CURRENT_LIBS   = self.VENV.get("libs", {})
+        self.CURRENT_ENVARS = self.VENV.get("envars", {})
         vars_map = self.get_dict()
         for key in vars_map:
             os.environ["KIWI_" + key] = vars_map[key]
 
         for key in self.CURRENT_MACROS:
             os.environ["KIWI_" + key] = self.CURRENT_MACROS[key]
-        self.dump_venv()
-        self.dump_config()
-        return True
+            
+        ret_ok = self.dump_venv() and self.dump_config()
+        self.print_current_envars_exporter()
+        return ret_ok
+    
+    def print_current_envars_exporter(self, change_ps1=True):
+
+        if not utils.shellposix:
+            utils.vprint("print_current_envars_exporter do not work, because the shellposix is False")
+            return
+
+        if "KIWI_PS1" in os.environ:
+            kiwi_ps1 = os.environ["KIWI_PS1"]
+            if kiwi_ps1 in ["0", "false", "False", "no"]:
+                change_ps1 = False
+
+        if change_ps1:
+            if "KIWI_OLD_PS1" in os.environ:
+                old_ps1 = os.environ["KIWI_OLD_PS1"]
+                match = re.search("\([a-zA-Z0-9\.\-\_]+\)", old_ps1)
+                if match:
+                    s, e = match.span(0)
+                    conda_name = old_ps1[s:e]
+                    new_name = conda_name + " [\\033[01;32m" + self.CURRENT_ENV + "\\033[0m]"
+                    while e < len(old_ps1) and old_ps1[e] == " ":
+                        e += 1
+                    new_ps1  = old_ps1[:s] + new_name + old_ps1[e:]
+                else:
+                    new_ps1 = "[\\033[01;32m" + self.CURRENT_ENV + "\\033[0m]" + old_ps1
+            else:
+                new_ps1 = "[\\033[01;32m" + self.CURRENT_ENV + "\\033[0m]" + "\\[\\033[01;32m\\]\\u@\\h\\[\\033[00m\\]:\\[\\033[01;34m\\]\\w\\[\\033[00m\\]\\$ "
+            print(f"export PS1=\"{new_ps1}\"")
+
+        for key in ["PATH", "LD_LIBRARY_PATH"]:
+            all_paths = []
+            if key in self.CURRENT_ENVARS:
+                PATHs = self.CURRENT_ENVARS[key]
+                if len(PATHs) > 0:
+                    utils.vprint(f"Register enviroment variable {key}.")
+                    for libname in PATHs:
+                        libPATH = PATHs[libname]
+                        all_paths.extend(utils.parse_weight_path(libPATH))
+
+            all_paths.append([100, f"$KIWI_OLD_{key}"])
+            all_paths = sorted(all_paths, key=lambda x:x[0])
+            cmd = ":".join([item[1] for item in all_paths])
+            print(f"export {key}={cmd}")
+
     
     def check_package(self, name, version_min=None, version_max=None):
 
@@ -236,7 +291,7 @@ class Config:
 
 class Actions:
     def __init__(self, app):
-        self.app = app
+        self.app : Application = app
         self.cfg : Config = app.cfg
 
     def private_run_cmd(self, args, _addi_args):
@@ -258,6 +313,40 @@ class Actions:
             utils.vprint(f"Remove directory: {dir}")
             shutil.rmtree(dir)
 
+    def __run_requirement(self, file):
+        utils.vprint(f"Run requirement script file: {file} at {os.getcwd()}")
+        try:
+            lines = open(file, "r").read().split("\n")
+            for line in lines:
+                line = line.strip()
+                if line.startswith("#") or line == "":
+                    continue
+
+                if line.startswith("@"):
+                    if len(line) < 3: return False
+                    utils.vprint(f"Run data command: {line[1:]}")
+                    if not self.app.run_with_command(f"getd {line[1:]}"):
+                        return False
+                elif line.startswith("$"):
+                    if len(line) < 3: return False
+                    utils.vprint(f"Run repo command: {line[1:]}")
+                    if not self.app.run_with_command(f"get {line[1:]}"):
+                        return False
+                elif line == "repvar":
+                    utils.vprint(f"Run install command: {line}")
+                    if not self.app.run_with_command(line):
+                        return False
+                elif line.startswith(">"):
+                    utils.gprint(utils.text_format_to_color(line[1:]))
+                else:
+                    utils.vprint(f"Run install command: {line}")
+                    if not self.app.run_with_command(f"install {line}"):
+                        return False
+            return True
+        except Exception as e:
+            traceback.print_exc()
+        return False
+
     def __run_py(self, file, args):
         utils.vprint(f"Run python script file: {file} at {os.getcwd()}")
 
@@ -271,6 +360,8 @@ class Actions:
             m = __import__(code_name, globals(), locals(), ["*"])
             utils.remove(temp_code_file)
             result =  getattr(m, "run")(self.app, args)
+            ddir = os.path.join(code_dir, "__pycache__")
+            self.__rmtree(ddir)
         except Exception as e:
             traceback.print_exc()
             result = False
@@ -353,6 +444,13 @@ class Actions:
 
     def __atomatic_run(self, work_dir, args):
         with ChangeCWD(work_dir):
+            # auto_config_file = "kiwi.required"
+            # if os.path.exists(auto_config_file):
+            #     utils.vprint(f"Run automatic script {auto_config_file}")
+            #     return self.__run_requirement(auto_config_file)
+            # else:
+            #     utils.vprint(f"The non-existent file {auto_config_file}")
+
             auto_config_file = ".kiwi/auto.py"
             if os.path.exists(auto_config_file):
                 utils.vprint(f"Run automatic script {auto_config_file}")
@@ -369,7 +467,7 @@ class Actions:
                 utils.vprint(f"The non-existent file {auto_config_file}")
         return True
 
-    def __register_lib(self, folder_name, lib_file, update_installed_lib=True):
+    def __register_lib(self, folder_name, lib_file):
         try:
             this_prefix = os.path.dirname(os.path.realpath(lib_file))
             lines = open(lib_file, "r").readlines()
@@ -394,21 +492,35 @@ class Actions:
             lib_name = attributes["NAME"]
             version  = attributes["VERSION"]
             description = attributes.get("DESCRIPTION", "")
+            if "PATH" not in self.cfg.CURRENT_ENVARS:
+                self.cfg.CURRENT_ENVARS["PATH"] = {}
 
-            if update_installed_lib:
-                utils.vprint(f"Add new library {lib_name} {version}")
-                self.cfg.INSTALLED_LIBS[folder_name] = {"name": lib_name, "version": version, "description": description}
-                if not self.cfg.dump_installed_libs():
-                    return False
-            
-            self.cfg.CURRENT_LIBS[lib_name] = [version, folder_name, description]
-            if not self.cfg.dump_venv():
+            if "LD_LIBRARY_PATH" not in self.cfg.CURRENT_ENVARS:
+                self.cfg.CURRENT_ENVARS["LD_LIBRARY_PATH"] = {}
+
+            PATHvalue = attributes.get("PATH", "")
+            if PATHvalue != "":
+                self.cfg.CURRENT_ENVARS["PATH"][lib_name] = PATHvalue
+
+            LDLIBRARYvalue = attributes.get("LD_LIBRARY_PATH", "")
+            if LDLIBRARYvalue != "":
+                self.cfg.CURRENT_ENVARS["LD_LIBRARY_PATH"][lib_name] = LDLIBRARYvalue
+
+            utils.vprint(f"Add new library {lib_name} {version}")
+            self.cfg.INSTALLED_LIBS[folder_name] = {"name": lib_name, "version": version, "description": description}
+            if not self.cfg.dump_installed_libs():
                 return False
 
+            self.cfg.CURRENT_LIBS[lib_name] = [version, folder_name, description]
             for key in attributes:
-                if key not in ["NAME", "VERSION", "DESCRIPTION"]:
+                if key not in ["NAME", "VERSION", "DESCRIPTION", "PATH", "LD_LIBRARY_PATH"]:
                     utils.vprint(f"Register macro: {key} = {attributes[key]}")
                     self.cfg.CURRENT_MACROS[key] = attributes[key]
+
+            if not self.cfg.dump_venv():
+                return False
+            
+            self.cfg.print_current_envars_exporter()
             return self.cfg.dump_venv()
         except:
             utils.gprint(f"Failed to register lib {folder_name}")
@@ -467,7 +579,16 @@ class Actions:
         http.extract_zip_to(file, proj, utils.verbose)
 
         if not args.disable_run:
-            self.__atomatic_run(proj, args)
+            result = self.__atomatic_run(proj, args)
+            endtips = None
+            if os.path.exists(".kiwi.endtips"):
+                utils.vprint("Remove endtips file: .kiwi.endtips")
+                endtips = open(".kiwi.endtips", "r").read()
+                os.remove(".kiwi.endtips")
+
+            if endtips is not None:
+                utils.gprint(endtips)
+            return result
         else:
             utils.vprint(f"Do not run any automatic scripts, because disable_run is set.")
         return True
@@ -519,6 +640,13 @@ class Actions:
 
     def install(self, args : argparse.Namespace, addi_args):
 
+        if args.pkg is None:
+            if os.path.exists("kiwi.required"):
+                return self.__run_requirement("kiwi.required")
+            else:
+                utils.gprint("The command 'install' need a pkg augment.")
+                return False
+
         pkg = args.pkg
         if pkg.find("/") == -1:
             utils.gprint(f"Invalid pkg name: {pkg}")
@@ -545,7 +673,7 @@ class Actions:
             if not args.force:
                 lib_file = os.path.abspath(os.path.join(install_to, ".kiwi.lib"))
                 if os.path.exists(lib_file):
-                    self.__register_lib(dname, lib_file, False)
+                    self.__register_lib(dname, lib_file)
                 utils.gprint(f"Package {dname} is already installed. Configured it to the current environment {self.cfg.CURRENT_ENV}")
                 utils.vprint(f"[install {pkg}] Since the installation directory already exists and is registered, we do not do anything without force")
                 return True
@@ -602,9 +730,9 @@ class Actions:
 
         utils.gprint(f"{len(self.cfg.CURRENT_LIBS)} libraries found:")
         for i, key in enumerate(self.cfg.CURRENT_LIBS):
-            name, version, descriptin = self.cfg.CURRENT_LIBS[key]
-            path = os.path.join(self.cfg.LIB_DIR, key)
-            utils.gprint(f"{i+1}. {name}-{version}\t{descriptin}\t{path}")
+            version, folder_name, descriptin = self.cfg.CURRENT_LIBS[key]
+            path = os.path.join(self.cfg.LIB_DIR, folder_name)
+            utils.gprint(f"{i+1}. {key}-{version}\t{descriptin}\t{path}")
         return True
 
     def macros(self, args : argparse.Namespace, addi_args):
@@ -618,6 +746,44 @@ class Actions:
     def activate(self, args : argparse.Namespace, addi_args):
         return self.cfg.activate_env(args.name)
     
+    def init(self, args : argparse.Namespace, addi_args):
+
+        os.makedirs(os.path.dirname(__file__), exist_ok=True)
+        kiwi_sh = os.path.join(os.path.dirname(__file__), "bin/kiwi.sh")
+        kiwi_sh_temp = os.path.join(os.path.dirname(__file__), "bin/kiwi.sh-temp")
+
+        bashrc_token_begin = "# >>> kiwi initialize >>>"
+        bashrc_token_end   = "# <<< kiwi initialize <<<"
+
+        templ_bashrc = f'''{bashrc_token_begin}
+# !! Contents within this block are managed by 'kiwi init' !!
+if [ -f \"{kiwi_sh}\" ]; then
+    . \"{kiwi_sh}\"
+    kiwi activate
+fi
+{bashrc_token_end}'''
+        bashrc_content = open(os.path.expanduser("~/.bashrc"), "r").read()
+        p0 = bashrc_content.find(bashrc_token_begin)
+        p1 = bashrc_content.find(bashrc_token_end)
+        if p0 != -1 and p1 != -1:
+            bashrc_content = bashrc_content[:p0] + templ_bashrc + bashrc_content[p1+len(bashrc_token_end):]
+        else:
+            bashrc_content += "\n" + templ_bashrc + "\n"
+
+        utils.vprint(f"Add specific scripts to bashrc to support kiwi running.")
+        open(os.path.expanduser("~/.bashrc"), "w").write(bashrc_content)
+        utils.gprint("The initialization is complete and you can restart the terminal to take effect the corresponding changes.")
+        
+        kiwi_sh_lines = open(kiwi_sh_temp, "r").read().split("\n")
+        kiwi_exe = sys.argv[0]
+        dst_value = f"export KIWI_EXE='{kiwi_exe}'"
+        kiwi_sh_lines[0] = dst_value
+
+        utils.vprint(f"Change the kiwi.sh line:0 to: {kiwi_sh_lines[0]}")
+        open(kiwi_sh, "w").write("\n".join(kiwi_sh_lines))
+        self.cfg.print_current_envars_exporter()
+        return True
+    
     def create(self, args : argparse.Namespace, addi_args):
         return self.cfg.create_env(args.name, args.copyfrom)
     
@@ -687,7 +853,13 @@ class Actions:
         file    = args.runfile
         workdir = args.workdir
 
+        already_run_required = False
         if file is None:
+            if os.path.exists("kiwi.required"):
+                already_run_required = self.__run_requirement("kiwi.required")
+                if not already_run_required:
+                    return False
+            
             file = os.path.abspath(".kiwi/auto.py")
             if not os.path.exists(file):
                 cwd = os.path.abspath(os.getcwd())
@@ -710,6 +882,7 @@ class Actions:
                         p = cwd.rfind("/", 0, p)
 
         if not os.path.exists(file):
+            if already_run_required: return True
             utils.gprint("Can not found any auto script to run.")
             return False
 
@@ -736,11 +909,12 @@ class Actions:
                 args.args = args_list
                 args.__dict__.update(kwargs_map)
 
+            ret = True
             if file.endswith(".py"):
-                return self.__run_py(file, args)
+                ret = self.__run_py(file, args)
             elif file.endswith(".sh"):
-                return self.__run_bash(file)
-            return False
+                ret = self.__run_bash(file)
+            return ret
 
     def search(self, args : argparse.Namespace, addi_args):
         param = urlencode({"key": args.key, "accessToken": self.cfg.ACCESS_TOKEN})
@@ -798,6 +972,7 @@ class Application:
         c.add_argument("-disable-run", action="store_true", help="Disable auto run")
 
         c = cmd.add_cmd("clean", "Clean cache files.")
+        c = cmd.add_cmd("init", "Initialize kiwi for terminal.")
         c = cmd.add_cmd("libs", "List all installed library.")
         c = cmd.add_cmd("macros", "List all installed library's macros.")
         c = cmd.add_cmd("repvar", "Macro variable replacement in the specified directory tree.")
@@ -806,7 +981,7 @@ class Application:
         c.add_argument("-inverse", action="store_true", help="inverse replace")
 
         c = cmd.add_cmd("activate", "Activate the current environment as specified")
-        c.add_argument("name", type=str, help="Virtual environment name")
+        c.add_argument("name", nargs="?", type=str, help="Virtual environment name")
 
         c = cmd.add_cmd("create", "Create the current environment as specified")
         c.add_argument("name", type=str, help="Virtual environment name")
@@ -815,7 +990,7 @@ class Application:
         c = cmd.add_cmd("envs", "Create the current environment as specified")
 
         c = cmd.add_cmd("install", "Install package from server")
-        c.add_argument("pkg", type=str, help="package name")
+        c.add_argument("pkg", nargs="?", type=str, help="package name")
         c.add_argument("-update", action="store_true", help="Force update")
         c.add_argument("-save", type=str, help="Unzipped folder")
         c.add_argument("-normtree", action="store_true", help="Unzip the data after remove the save folder.")
@@ -838,11 +1013,22 @@ class Application:
         elif args is None:
             args = sys.argv[1:]
 
+        # remove space
+        for i in range(len(args)-1, -1, -1):
+            if args[i].strip() == "":
+                del args[i]
+
         for i in range(len(args)):
             if args[i] == "-verbose":
                 del args[i]
                 utils.verbose = True
                 break
+                
+        for i in range(len(args)):
+            if args[i] == "-shell.posix":
+                del args[i]
+                utils.shellposix = True
+                break
         
         addi_args = None
         if len(args) > 0:
