@@ -5,6 +5,7 @@ import hashlib
 import zipfile
 from tqdm import tqdm
 import json
+from . import utils
 
 def compute_file_md5(file):
 
@@ -37,7 +38,7 @@ def request_text(url):
     if jdict["status"] == "success":
         return jdict["data"]
     
-    print("Request text failed: ", jdict["message"])
+    utils.gprint("Request text failed: ", jdict["message"])
     return None
 
 
@@ -48,11 +49,11 @@ def request_file(url, file, title):
         response   = requests.get(url, stream=True)
         if response.status_code != 200:
             if response.status_code == 404:
-                print(f"Not found.")
+                utils.gprint(f"Not found.")
                 return False, "NotFound"
 
             jdict = json.loads(response.content)
-            print(f"Download failed: {jdict['message']}")
+            utils.gprint(f"Download failed: {jdict['message']}")
             return False, "Failed"
 
         content_iter = response.iter_content(chunk_size=chunk_kb_size)
@@ -83,8 +84,11 @@ def request_data(url):
     try:
         response   = requests.get(url, stream=True)
         if response.status_code != 200:
-            jdict = json.loads(response.content)
-            print(f"Request failed: {jdict['message']}")
+            if response.headers["Content-Type"].find("application/json") != -1:
+                jdict = json.loads(response.content)
+                utils.gprint(f"Request failed: {jdict['message']}")
+            else:
+                utils.gprint(f"Request failed: {response.content}")
             return None
 
         return response.content
@@ -99,21 +103,25 @@ def require_file_and_check_md5(url, md5url, file, title, update=False, verbose=F
         os.makedirs(root_dir, exist_ok=True)
 
         if os.path.exists(file) and not update:
-            if verbose:
-                print(f"File already download at {file}")
-            return True, json.load(open(file + ".info", "r")), "Cached"
+            meta = json.load(open(file + ".info", "r"))
+            if os.path.getsize(file) == meta["size"]:
+                utils.vprint(f"File already download at {file}")
+                return True, meta, "Cached"
+            else:
+                utils.gprint(f"Found a broken package file {file}, I will remove it.")
+                os.remove(file)
 
         if verbose:
-            print("Send an http request to check the file has been updated...")
+            utils.gprint("Send an http request to check the file has been updated...")
 
         remote_info = request_text(md5url)
         if remote_info is None:
             if verbose:
-                print(f"Can not request remote info from url: {md5url}")
+                utils.gprint(f"Can not request remote info from url: {md5url}")
             return False, None, "Failed"
 
         if verbose:
-            print(f"Dump info to: {file}.info")
+            utils.gprint(f"Dump info to: {file}.info")
 
         json.dump(remote_info, open(file + ".info", "w"), ensure_ascii=True, indent=4)
         if os.path.exists(file):
@@ -122,7 +130,7 @@ def require_file_and_check_md5(url, md5url, file, title, update=False, verbose=F
             if remote_md5 == local_md5:
                 # MD5 matched
                 if verbose:
-                    print(f"File already download at {file}, because the MD5 is matched.")
+                    utils.gprint(f"File already download at {file}, because the MD5 is matched.")
                 return True, remote_info, "Cached"
 
         ok, state = request_file(url, file, title)
@@ -146,5 +154,5 @@ def extract_zip_to(file, to, verbose=True):
 
         zfile.extract(member, to)
         if verbose:
-            print(f"Extract file {os.path.join(to, member)}")
+            utils.gprint(f"Extract file {os.path.join(to, member)}")
     return members
\ No newline at end of file
