@@ -3,14 +3,20 @@ import re
 import subprocess
 import json
 import traceback
+import sys
 
 verbose = False
+shellposix = False
 
 def vprint(*args, **kwargs):
     if verbose:
+        if shellposix:
+            kwargs["file"] = sys.stderr
         print(*args, **kwargs)
 
 def gprint(*args, **kwargs):
+    if shellposix:
+        kwargs["file"] = sys.stderr
     print(*args, **kwargs)
 
 def get_python_link_name(pydll_path, os_name):
@@ -68,14 +74,14 @@ def version_to_int(version):
         vprint(f"Ignore numbers after 4 digits of the version number, {version}")
         array = array[:4]
     
-    multipliers = [1<<24, 1<<16, 1<<8, 1]
+    multipliers = [1<<48, 1<<32, 1<<16, 1]
     imultiplier = 0
     output_number = 0
     for value in array:
         value = int(re.sub("\D", "", value))
-        if value > 255:
-            vprint(f"Numbers larger than 255 are trimmed to within 255, when {value} > 255")
-            value = 255
+        if value > (1 << 16):
+            vprint(f"Numbers larger than {1 << 16} are trimmed to within {1 << 16}, when {value} > {1 << 16}")
+            value = 1 << 16
 
         output_number += value * multipliers[imultiplier]
         imultiplier += 1
@@ -194,4 +200,36 @@ def replace_file_with_var_map(file, vars_dict, inverse=False):
     except:
         traceback.print_exc()
         vprint(f"Can not replace variable in file: {file}")
-    return False
\ No newline at end of file
+    return False
+
+
+# [1]${this}/lib:[2]${this}/lib2
+def parse_weight_path(paths, default_weight=10):
+    weight_items = []
+    for item in paths.split(":"):
+        if item.startswith("["):
+            p = item.find("]")
+            if p == -1:
+                weight_items.append([default_weight, item])
+                continue
+
+            weight_items.append([int(item[1:p]), item[p+1:]])
+            continue
+        weight_items.append([default_weight, item])
+    return weight_items
+
+def text_format_to_color(text):
+    # <red>You can execute 'make run' to run this project.</red>
+    text = text.replace("<red>", "\033[31m")
+    text = text.replace("</red>", "\033[0m")
+    text = text.replace("<green>", "\033[32m")
+    text = text.replace("</green>", "\033[0m")
+    text = text.replace("<yellow>", "\033[33m")
+    text = text.replace("</yellow>", "\033[0m")
+    text = text.replace("<blue>", "\033[34m")
+    text = text.replace("</blue>", "\033[0m")
+    text = text.replace("<mag>", "\033[35m")
+    text = text.replace("</mag>", "\033[0m")
+    text = text.replace("<cyan>", "\033[36m")
+    text = text.replace("</cyan>", "\033[0m")
+    return text
\ No newline at end of file
